package org.eclipse.hyades.logging.events;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectStreamField;
import java.io.Serializable;

/**********************************************************************
 * Copyright (c) 2005 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: RequestSituationImpl.java,v 1.5 2005/02/16 22:20:48 qiyanli Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

/**
 * The RequestSituation deals with the situations that a component uses to identify 
 * the completion status of a request. Typically, these requests are complex management 
 * tasks or transactions that a component undertakes on behalf of a requestor and not 
 * the mainline simple requests or transactions. Existing messages include words like 
 * configuration synchronization started, and backup procedure complete, for example:
 * 
 * <code>
 * ADMS0003I: Configuration synchronization completed
 * </code>
 * 
 * 
 * @author Richard K. Duggan
 * @author Paul E. Slauenwhite
 * @version 1.0.1
 * @since 1.0.1
 * @see IStartSituation
 * @see IRequestSituation
 * @deprecated This class is deprecated as of 2004-05-17. Use the corresponding class available in org.eclipse.hyades.logging.events.cbe*
 */
public class RequestSituationImpl extends SituationTypeImpl implements IRequestSituation, Serializable, Cloneable {

    //Define the serialiVersionUID for serialization:
    //NOTE: If this object is ever changed which breaks backward compatibility,
    //this value MUST be changed.  Every effort should be made not to break 
    //backward compatibility.
    static final long serialVersionUID = -3888096669772662967L;

    //Define the fields that will be serialized: 
    //NOTE: If any new variables are added to the class this table should be 
    //updated along with the readObject and writeObject methods.
    private static final ObjectStreamField[] serialPersistentFields = { new ObjectStreamField("successDisposition", String.class), new ObjectStreamField("situationQualifier", String.class)};

    //Class instance variable(s):
    protected String successDisposition = null;
    protected String situationQualifier = null;

    /**
     * No-argument protected constructor for RequestSituationImpl.
     */
    protected RequestSituationImpl() {
		super();
    }

    /**
     * Returns the successDisposition.
     * 
     * This property specifies the success disposition of an operation of a 
     * situation that caused the situation to be reported.  The successDisposition 
     * is a string with the following set of values:
     * 
     * SUCCESSFUL
     * UNSUCESSFUL
     * 
     * This is a REQUIRED property and once it is set it MUST NOT change. 
     * The string length for this property MUST NOT exceed 64 characters.
     * 
     * @return String The returned successDisposition.
     * @since 1.0.1
     */
    public String getSuccessDisposition() {
        return successDisposition;
    }

    /**
     * Returns the situationQualifier.
     * 
     * This property specifies the request qualifiers that are representation of 
     * the parameters necessary to describe the  situation.  The situationQualifier 
     * is a string with the following set of values:
     * 
     * REQUEST INITIATED
     * REQUEST COMPLETED
     * 
     * This is a REQUIRED property and once it is set it MUST NOT change. 
     * The string length for this property MUST NOT exceed 64 characters.
     * 
     * @return String The returned situationQualifier.
     * @since 1.0.1
     */
    public String getSituationQualifier() {
        return situationQualifier;
    }

    /**
     * Sets the successDisposition.
     * 
     * This property specifies the success disposition of an operation of a 
     * situation that caused the situation to be reported.  The successDisposition 
     * is a string with the following set of values:
     * 
     * SUCCESSFUL
     * UNSUCESSFUL
     * 
     * This is a REQUIRED property and once it is set it MUST NOT change. 
     * The string length for this property MUST NOT exceed 64 characters.
     * 
     * @param successDisposition The successDisposition to be set.
     * @since 1.0.1
     */
    public void setSuccessDisposition(String successDisposition) {
        this.successDisposition = successDisposition;
    }

    /**
     * Sets the situationQualifier.
     * 
     * This property specifies the request qualifiers that are representation of 
     * the parameters necessary to describe the  situation.  The situationQualifier 
     * is a string with the following set of values:
     * 
     * REQUEST INITIATED
     * REQUEST COMPLETED
     * 
     * This is a REQUIRED property and once it is set it MUST NOT change. 
     * The string length for this property MUST NOT exceed 64 characters.
     * 
     * @param situationQualifier The situationQualifier to be set.
     * @since 1.0.1
     */
    public void setSituationQualifier(String situationQualifier) {
        this.situationQualifier = situationQualifier;
    }

    /**
     * Overrides the <code>java.lang.Object</code>'s equals() API to 
     * determine if the parameter <code>object</code> is equal to this object.
     * 
     * @param object The <code>java.lang.Object</code> to be compared to this object.
     * @return true if this object is the same as the parameter <code>object</code>, false otherwise.
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object object) {

        //Check if the parameter object's super class is equal to the this object's super class:
        if (super.equals(object)) {

            RequestSituationImpl requestSituationImpl = ((RequestSituationImpl) (object));

            //Compare the value(s) of all object properties:
            String thisObjectsProperty = getSuccessDisposition();
            String parameterObjectsProperty = requestSituationImpl.getSuccessDisposition();

            if (((thisObjectsProperty != null) && (thisObjectsProperty.equals(parameterObjectsProperty))) || ((thisObjectsProperty == null) && (parameterObjectsProperty == null))) {

                thisObjectsProperty = getSituationQualifier();
                parameterObjectsProperty = requestSituationImpl.getSituationQualifier();

                if (((thisObjectsProperty != null) && (thisObjectsProperty.equals(parameterObjectsProperty))) || ((thisObjectsProperty == null) && (parameterObjectsProperty == null)))
                    return true;
            }
        }

        return false;
    }
    private void writeObject(ObjectOutputStream out) throws IOException {

        //Set the values of the serializable field(s):
        ObjectOutputStream.PutField fields = out.putFields();

        fields.put("successDisposition", successDisposition);
        fields.put("situationQualifier", situationQualifier);

        //Write the values of the serializable field(s):
        out.writeFields();
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {

        //Prepare and read the alternate persistent field(s):
        ObjectInputStream.GetField fields = null;

        fields = in.readFields();

        //Retrieve the alternate persistent field(s):
        successDisposition = ((String) (fields.get("successDisposition", successDisposition)));
        situationQualifier = ((String) (fields.get("situationQualifier", situationQualifier)));
    }

    /**
     * Return a clone (copy by value) of this object. This is a deep copy version, 
     * in which all the objects within this object will be copied by value.
     *  
     * @return Object The clone of this object.
     */
    public Object clone() throws CloneNotSupportedException {
        return (super.clone());
    }

    /**
     * Resets the object's properties to their initial (e.g. null) state.
     * 
     * @see org.eclipse.hyades.logging.events.ISituationType#init()
     */
    public void init() {

        super.init();

        successDisposition = null;
        situationQualifier = null;
    }
}